@@ -44,22 +44,27 @@ float *pyvector_to_Carrayptrs(PyArrayObject *arrayin)
 }
 
 static PyObject *
-kcorrect_load_templates(PyObject *self, PyObject *args)
+_kcorrect_load_templates(PyObject *self, PyObject *args)
 {
     char * vfile;
     char * lfile;
+    IDL_LONG k;
     if (!PyArg_ParseTuple(args, "ss", &vfile, &lfile))
         return NULL;
 
     /* read in templates */
-    k_read_ascii_table(&vmatrix,&ndim,&sizes,vfile);
+    k = k_read_ascii_table(&vmatrix,&ndim,&sizes,vfile);
+    if ((k!=1) && PyErr_Occurred()) {
+	PyErr_SetString( _kcorrectError,"unable to read vmatrix template.\n");
+	return NULL;
+    }
     nl=sizes[1];
     nv=sizes[0];
     FREEVEC(sizes);
     k_read_ascii_table(&lambda,&ndim,&sizes,lfile);
-    if ((sizes[0]!=nl+1)  && PyErr_Occurred()) {
-            PyErr_SetString( _kcorrectError,"incompatible vmatrix and lambda files.\n");
-            return NULL;;
+    if ((sizes[0]!=nl+1) && PyErr_Occurred()) {
+        PyErr_SetString( _kcorrectError,"incompatible vmatrix and lambda files.\n");
+        return NULL;
     } /* end if */
     FREEVEC(sizes);
 
@@ -67,12 +72,12 @@ kcorrect_load_templates(PyObject *self, PyObject *args)
     return Py_None;
 }
 
-PyDoc_STRVAR(kcorrect_load_templates_doc,
+PyDoc_STRVAR(_kcorrect_load_templates_doc,
 "loads the templates."
 );
 
 static PyObject *
-kcorrect_load_filters(PyObject *self, PyObject *args)
+_kcorrect_load_filters(PyObject *self, PyObject *args)
 {
     char * ffile;
     int i;
@@ -100,12 +105,157 @@ kcorrect_load_filters(PyObject *self, PyObject *args)
     return Py_None;
 }
 
-PyDoc_STRVAR(kcorrect_load_filters_doc,
+PyDoc_STRVAR(_kcorrect_load_filters_doc,
 "loads the filters."
 );
 
 static PyObject *
-kcorrect_fit_coeffs_from_file(PyObject *self, PyObject *args)
+_kcorrect_projection_table(PyObject *self, PyObject *args)
+{
+    PyArrayObject *py_lambda, *py_vmatrix, *py_rmatrix=NULL;
+    npy_intp *shape;
+    PyObject *newshape, *_iter = NULL;
+    float *_lambda, *_vmatrix;
+    float *_rmatrix=NULL;
+    float *_zvals=NULL;
+    IDL_LONG _nk,_nv,_nl;
+    float *_filter_lambda=NULL;
+    float *_filter_pass=NULL;
+    IDL_LONG *_filter_n=NULL;
+    IDL_LONG _maxn, _nz;
+    char * ffile;
+    int i;
+    float band_shift, _zmin, _zmax;
+    npy_intp *dims=NULL;
+    PyArray_Descr * dsc;
+    if (!PyArg_ParseTuple(args, "O!O!fsffi",
+			  &PyArray_Type, &py_lambda,
+			  &PyArray_Type, &py_vmatrix,
+			  &band_shift, &ffile, 
+			  &_zmin, &_zmax, &_nz))
+        return NULL;
+    ndim = PyArray_NDIM(py_vmatrix); /*  ndim global !! not needed here*/
+    shape = PyArray_DIMS(py_vmatrix); /*PyArray_SHAPE(py_vmatrix); */
+    _nv = shape[0];
+    _nl = shape[1];
+    newshape = Py_BuildValue("(i)", _nv*_nl);
+    _lambda = pyvector_to_Carrayptrs(py_lambda);
+    _vmatrix = pyvector_to_Carrayptrs((PyArrayObject *)PyArray_Reshape(py_vmatrix, newshape));
+    /* todo: check if filters are loaded */
+    k_load_filters(&_filter_n,&_filter_lambda,&_filter_pass,&_maxn,&_nk,ffile);
+    _rmatrix=(float *) malloc(_nz*_nv*_nk*sizeof(float));
+    dims=(npy_intp *) malloc(sizeof(npy_intp));    
+    dims[0] = _nz*_nv*_nk;
+    dsc = PyArray_DescrFromType(NPY_FLOAT);
+    _zvals=(float *) malloc(_nz*sizeof(float));
+    for(i=0;i<_nz;i++)
+        _zvals[i]=_zmin+(_zmax-_zmin)*((float)i+0.5)/(float)_nz;
+     
+    k_projection_table(_rmatrix,_nk,_nv,_vmatrix,_lambda,_nl,_zvals,_nz,_filter_n,
+                       _filter_lambda,_filter_pass,band_shift,_maxn);
+
+    py_rmatrix = (PyArrayObject *) PyArray_Zeros(1, dims, dsc, 0);
+    if (NULL == py_rmatrix)  goto fail;
+    _iter = PyArray_IterNew((PyObject*)py_rmatrix);
+    if (_iter == NULL) goto fail;
+    for(i=0;i<_nz*_nv*_nk;i++){
+        if (PyArray_SETITEM(py_rmatrix, PyArray_ITER_DATA(_iter), PyFloat_FromDouble(_rmatrix[i]))) {
+            PyErr_SetString(_kcorrectError, "unable to set _rmatrix");
+            goto fail;
+         }
+         PyArray_ITER_NEXT(_iter);
+    }
+
+    FREEVEC(_rmatrix);
+    FREEVEC(_zvals);
+    FREEVEC(dims);
+    Py_DECREF(_iter);
+    Py_INCREF(py_rmatrix);
+    return PyArray_Return(py_rmatrix);
+fail:
+    Py_XDECREF(_iter);
+    Py_XDECREF(py_rmatrix);
+    return NULL;
+}
+
+PyDoc_STRVAR(_kcorrect_projection_table_doc,
+"create the lookup table for given wavelength and flux."
+);
+static PyObject *
+_kcorrect_fit_nonneg(PyObject *self, PyObject *args)
+{
+    /* rmatrix,nk,nv,zvals,nz should be already set with load_filters */
+
+    PyArrayObject *py_maggies, *py_maggies_ivar, *py_coeffs=NULL;
+    float r, *c_maggies, *c_maggies_ivar, *c_coeffs;
+    int i;
+    IDL_LONG niter;
+    npy_intp dims[] = {nv};
+    PyArray_Descr * dsc;
+    dsc = PyArray_DescrFromType(NPY_FLOAT32);
+    if (NULL == vmatrix || NULL == lambda ) { 
+        PyErr_SetString( _kcorrectError,"no templates loaded.\n");
+        return NULL;;
+    } /* end if */
+    if (NULL == rmatrix) { 
+        PyErr_SetString( _kcorrectError,"no filters loaded.\n");
+        return NULL;;
+    } /* end if */
+    if (!PyArg_ParseTuple(args, "fO!O!",
+			  &r,
+			  &PyArray_Type, &py_maggies,
+			  &PyArray_Type, &py_maggies_ivar))
+        return NULL;
+    if ((PyArray_DIMS(py_maggies)[0] != nk) || (PyArray_DIMS(py_maggies_ivar)[0] != nk)) {
+        PyErr_SetString( _kcorrectError,"maggies dimension not compatible with number of filters.");
+	return NULL;
+    }
+    /* free previous coeffs and maggies*/
+    if (NULL != coeffs) FREEVEC(coeffs);
+    if (NULL != chi2) FREEVEC(chi2);
+    if (NULL != redshift) FREEVEC(redshift);
+    if (NULL != maggies) FREEVEC(maggies);
+    if (NULL != maggies_ivar) FREEVEC(maggies_ivar);
+    /* and malloc for new ones */
+    redshift=(float *) malloc(sizeof(float));
+    maggies=(float *) malloc(nk*sizeof(float));
+    maggies_ivar=(float *) malloc(nk*sizeof(float));
+    coeffs=(float *) malloc(nv*sizeof(float));
+    chi2=(float *) malloc(sizeof(float));
+
+    c_maggies = pyvector_to_Carrayptrs(py_maggies);
+    c_maggies_ivar = pyvector_to_Carrayptrs(py_maggies_ivar);
+    for (i=0; i<nk; i++) {
+        maggies[i] = c_maggies[i];
+	maggies_ivar[i] = c_maggies_ivar[i];
+	}
+    redshift[0] = r;
+
+    k_fit_nonneg(coeffs,rmatrix,nk,nv,zvals,nz,
+		      c_maggies,c_maggies_ivar,redshift,
+		      1,tolerance,maxiter,&niter,chi2,0,0);
+  
+
+    py_coeffs = (PyArrayObject *) PyArray_NewFromDescr(&PyArray_Type,
+                                                 dsc,
+                                                 1,
+                                                 dims,
+                                                 NULL,
+                                                 NULL,
+                                                 0,
+                                                 NULL);
+    if (NULL == py_coeffs)  return NULL;
+    c_coeffs=pyvector_to_Carrayptrs(py_coeffs);
+    for(i=0;i<nv;i++) c_coeffs[i] = coeffs[i];
+    return PyArray_Return(py_coeffs);
+}
+
+PyDoc_STRVAR(_kcorrect_fit_nonneg_doc,
+"Given the rmatrix, does a nonnegative fit of templates to data."
+);
+
+static PyObject *
+_kcorrect_fit_coeffs_from_file(PyObject *self, PyObject *args)
 {
     IDL_LONG i,j,k,niter,nchunk,ncurrchunk;
     float *mag;
@@ -172,13 +322,13 @@ kcorrect_fit_coeffs_from_file(PyObject *self, PyObject *args)
     return Py_None;
 }
 
-PyDoc_STRVAR(kcorrect_fit_coeffs_from_file_doc,
-"writes the computed coeffs to a file\n\
+PyDoc_STRVAR(_kcorrect_fit_coeffs_from_file_doc,
+"writes the computed coeffs into a file\n\
 with redshift and maggies read from a given file"
 );
 
 static PyObject *
-kcorrect_fit_coeffs(PyObject *self, PyObject *args)
+_kcorrect_fit_coeffs(PyObject *self, PyObject *args)
 {
     IDL_LONG j,niter;
     PyArrayObject *pyin, *pyout;
@@ -247,12 +397,12 @@ kcorrect_fit_coeffs(PyObject *self, PyObject *args)
     return PyArray_Return(pyout);                     
 }
 
-PyDoc_STRVAR(kcorrect_fit_coeffs_doc,
+PyDoc_STRVAR(_kcorrect_fit_coeffs_doc,
 "computes the coeffs from given maggies and redshift"
 );
 
 static PyObject *
-kcorrect_reconstruct_maggies(PyObject *self, PyObject *args)
+_kcorrect_reconstruct_maggies(PyObject *self, PyObject *args)
 {
     IDL_LONG j;
     PyArrayObject *pyin, *pyout;
@@ -276,7 +426,7 @@ kcorrect_reconstruct_maggies(PyObject *self, PyObject *args)
                           &PyArray_Type, &pyin,
                           &all_redshift))
         return NULL;
-    if (NULL == pyin)  return NULL;
+    //if (NULL == pyin)  return NULL;
     pyout=(PyArrayObject *) PyArray_NewFromDescr(&PyArray_Type,             
                                                  dsc,
                                                  1,
@@ -315,12 +465,12 @@ kcorrect_reconstruct_maggies(PyObject *self, PyObject *args)
     return PyArray_Return(pyout);                     
 }
 
-PyDoc_STRVAR(kcorrect_reconstruct_maggies_doc,
+PyDoc_STRVAR(_kcorrect_reconstruct_maggies_doc,
 "reconstructs maggies from given coeffs and redshift"
 );
 
 static PyObject *
-kcorrect_fit_photoz(PyObject *self, PyObject *args)
+_kcorrect_fit_photoz(PyObject *self, PyObject *args)
 {
     IDL_LONG j,niter;
     PyArrayObject *pyin, *pyout;
@@ -342,7 +492,7 @@ kcorrect_fit_photoz(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "O!",
                           &PyArray_Type, &pyin))
         return NULL;
-    if (NULL == pyin)  return NULL;
+    //   if (NULL == pyin)  return NULL;
     pyout=(PyArrayObject *) PyArray_NewFromDescr(&PyArray_Type,             
                                                  dsc,
                                                  1,
@@ -393,22 +543,86 @@ kcorrect_fit_photoz(PyObject *self, PyObject *args)
     return PyArray_Return(pyout);                     
 }
 
-PyDoc_STRVAR(kcorrect_fit_photoz_doc,
+PyDoc_STRVAR(_kcorrect_fit_photoz_doc,
 "returns guessed coeffs at redshift"
 );
 
+static PyObject *
+_kcorrect_template(PyObject *self, PyObject *args)
+{
+    float *v = NULL;
+    char *vfile;
+    PyArrayObject *pyout;
+    PyArray_Descr *dsc;
+    PyObject *pyout_iter = NULL;
+    npy_intp *dims=NULL;
+    IDL_LONG n, *s=NULL;
+    int i,j=1;
+    dsc = PyArray_DescrFromType(NPY_FLOAT);
+    if (!PyArg_ParseTuple(args, "s", &vfile))
+        return NULL;
+    k_read_ascii_table(&v,&n,&s,vfile);
+    dims = (npy_intp *) malloc((n)*sizeof(npy_intp));    
+    if (NULL == dims) return NULL;
+    for (i=0; i<n; i++) {
+        dims[i] = s[i];
+	j *= s[i];
+    }
+    pyout = (PyArrayObject *) PyArray_Zeros((npy_intp)n, dims, dsc, 0);
+    if (NULL == pyout)  goto fail;
+    pyout_iter = PyArray_IterNew((PyObject*)pyout);
+    if (pyout_iter == NULL) {
+        PyErr_SetString(_kcorrectError, "cannot create iterators");
+        goto fail;
+    }
+    for (i=0;i<j;i++) {
+      if (PyArray_SETITEM(pyout, PyArray_ITER_DATA(pyout_iter), PyFloat_FromDouble((double)v[i]))) {
+	    PyErr_SetString(_kcorrectError, "unable to set vmatrix");
+	    goto fail;
+	}
+        PyArray_ITER_NEXT(pyout_iter);
+    }
+    Py_DECREF(pyout_iter);
+    Py_INCREF(pyout);
+    return PyArray_Return(pyout);
+
+fail:
+    Py_XDECREF(pyout_iter);
+    Py_XDECREF(pyout);
+    return NULL;
+}
+
+PyDoc_STRVAR(_kcorrect_template_doc,
+"return vmatrix or lambda from a template file."
+);
+
+static PyObject *
+_kcorrect_band_shift(PyObject *self)
+{
+  return Py_BuildValue("d", zmin);
+}
+
+PyDoc_STRVAR(_kcorrect_band_shift_doc,
+"return constants."
+);
+
+
 static PyMethodDef kcorrect_methods[] = {
-    {"load_templates", kcorrect_load_templates, METH_VARARGS, kcorrect_load_templates_doc},
-    {"load_filters", kcorrect_load_filters, METH_VARARGS, kcorrect_load_filters_doc},
-    {"fit_coeffs_from_file", kcorrect_fit_coeffs_from_file, METH_VARARGS, kcorrect_fit_coeffs_from_file_doc},
-    {"fit_coeffs", kcorrect_fit_coeffs, METH_VARARGS, kcorrect_fit_coeffs_doc},
-    {"reconstruct_maggies", kcorrect_reconstruct_maggies, METH_VARARGS, kcorrect_reconstruct_maggies_doc},
-    {"fit_photoz", kcorrect_fit_photoz, METH_VARARGS, kcorrect_fit_photoz_doc},
+    {"load_templates", _kcorrect_load_templates, METH_VARARGS, _kcorrect_load_templates_doc},
+    {"load_filters", _kcorrect_load_filters, METH_VARARGS, _kcorrect_load_filters_doc},
+    {"fit_coeffs_from_file", _kcorrect_fit_coeffs_from_file, METH_VARARGS, _kcorrect_fit_coeffs_from_file_doc},
+    {"fit_coeffs", _kcorrect_fit_coeffs, METH_VARARGS, _kcorrect_fit_coeffs_doc},
+    {"reconstruct_maggies", _kcorrect_reconstruct_maggies, METH_VARARGS, _kcorrect_reconstruct_maggies_doc},
+    {"fit_photoz", _kcorrect_fit_photoz, METH_VARARGS, _kcorrect_fit_photoz_doc},
+    {"template", _kcorrect_template, METH_VARARGS, _kcorrect_template_doc},
+    {"projection_table", _kcorrect_projection_table, METH_VARARGS, _kcorrect_projection_table_doc},
+    {"fit_nonneg", _kcorrect_fit_nonneg, METH_VARARGS, _kcorrect_fit_nonneg_doc},
+    {"band_shift", (PyCFunction)_kcorrect_band_shift, METH_NOARGS, _kcorrect_band_shift_doc},
     {NULL,		NULL}		/* sentinel */
 };
 
 PyDoc_STRVAR(module_doc,
-"This module provides kcorrect"
+"This module provides Python wrapper for kcorrect library"
 );
 
 #if PY_VERSION_HEX >= 0x03000000
